package com.jarvis.dataCenter.runner;

import com.google.code.or.OpenReplicator;
import com.jarvis.common.redis.service.RedisService;
import com.jarvis.commons.open.replicator.Test;
import com.jarvis.commons.open.replicator.cache.CDCEventManager;
import com.jarvis.commons.open.replicator.event.CDCEvent;
import com.jarvis.commons.open.replicator.info.BinlogMasterStatus;
import com.jarvis.commons.open.replicator.listener.NotificationListener;
import com.jarvis.commons.open.replicator.util.MysqlConnection;
import com.jarvis.dataCenter.config.OpenReplicatorCofig;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.data.redis.RedisAutoConfiguration;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

/**
 * @author jinzhaopo
 * @date 2019-11-04 17:54
 */
@Service
@Data
@AutoConfigureAfter(OpenReplicatorCofig.class)
@Slf4j
public class OpenReplicatorRunner {

    public String binlogPositionRedisKey = "bin_log_position_redis_key";

    @Autowired
    private OpenReplicatorCofig openReplicatorCofig;

    @Autowired
    private RedisService redisService;

    public OpenReplicator openReplicator;

    public void initOpenReplicator() {
        MysqlConnection.setConnection(openReplicatorCofig.getHost(), Integer.valueOf(openReplicatorCofig.getPort()), openReplicatorCofig.getUser(), openReplicatorCofig.getPwd());
        BinlogMasterStatus bms = MysqlConnection.getBinlogMasterStatus();

        openReplicator = new OpenReplicator();
        openReplicator.setHost(openReplicatorCofig.getHost());
        openReplicator.setUser(openReplicatorCofig.getUser());
        openReplicator.setPassword(openReplicatorCofig.getPwd());
        openReplicator.setPort(Integer.valueOf(openReplicatorCofig.getPort()));
        openReplicator.setBinlogFileName(bms.getBinlogName());
        openReplicator.setBinlogEventListener(new NotificationListener());


    }


    public void start() {
        Long binlogPosition = (Long) redisService.get(binlogPositionRedisKey);
        if (binlogPosition == null) {
            BinlogMasterStatus bms = MysqlConnection.getBinlogMasterStatus();
            binlogPosition = bms.getPosition();
        }
        openReplicator.setBinlogPosition(binlogPosition);
        try {
            openReplicator.start();
        } catch (Exception e) {
            e.printStackTrace();
        }

        Thread thread = new Thread(new Test.PrintCDCEvent());
        thread.start();

    }

    public void stop() {
        try {
            openReplicator.stop(10, TimeUnit.SECONDS);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static class PrintCDCEvent implements Runnable {
        @Override
        public void run() {
            while (true) {
                if (CDCEventManager.queue.isEmpty() == false) {
                    CDCEvent ce = CDCEventManager.queue.pollFirst();


                    System.out.println(ce.toString());
                } else {
                    try {
                        TimeUnit.SECONDS.sleep(1);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }
}
