package com.jarvis.dataCenter.service.impl.report;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.jarvis.common.redis.service.RedisService;
import com.jarvis.dataCenter.bean.BeanUtils;
import com.jarvis.dataCenter.constants.DbConstants;
import com.jarvis.dataCenter.entity.report.ReportDoInfo;
import com.jarvis.dataCenter.entity.warehouse.*;
import com.jarvis.dataCenter.mapper.report.ReportDoInfoMapper;
import com.jarvis.dataCenter.mapper.warehouse.DeliveryOrderDetailMapper;
import com.jarvis.dataCenter.service.report.ReportDoInfoService;
import com.jarvis.dataCenter.service.warehouse.*;
import com.jarvis.lib.common.mybatisplus.service.impl.BaseServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author jinzhaopo
 * @date 2019-10-30 14:57
 */
@DS(DbConstants.REPORT)
@Service
@Slf4j
public class ReportDoInfoServiceImpl extends BaseServiceImpl<ReportDoInfoMapper, ReportDoInfo> implements ReportDoInfoService {


    @Autowired
    private DeliveryOrderService deliveryOrderService;

    @Autowired
    private DeliveryOrderDetailService deliveryOrderDetailService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private DeliveryOrderDetailMapper deliveryOrderDetailMapper;

    @Autowired
    private StockAllocationService stockAllocationService;

    @Autowired
    private StockContainerService stockContainerService;

    @Autowired
    private DeliveryLicenceService deliveryLicenceService;


    public static boolean SYNC = false;

    public static String REDIS_KEY = "sync:report:dodetail:id:";


    @Autowired
    private WarehouseService warehouseService;

    @Override
    public boolean sync() {
        Long endId = deliveryOrderDetailMapper.maxId();
        Integer startId = (Integer) redisService.get(REDIS_KEY);
        if (startId == null) {
            startId = 0;
        }
        if (startId.intValue() > endId.intValue()) {
            return false;
        }

        //  同步信息
        for (long i = startId.intValue(); i <= endId.intValue(); i++) {
            try {
                sync(i);
            } catch (Exception e) {
                e.printStackTrace();
            }
            redisService.getRedisTemplateObj().opsForValue().increment(REDIS_KEY, 1);
        }

        return true;
    }

    @Override
    public ReportDoInfo sync(Long doDetailId) {
        ReportDoInfo reportDoInfo = new ReportDoInfo();

        //发货单详情
        DeliveryOrderDetailEntity deliveryOrderDetailEntity = deliveryOrderDetailService.getById(doDetailId);
        BeanUtils.build(reportDoInfo, deliveryOrderDetailEntity);

        if (deliveryOrderDetailEntity == null) {
            return null;
        }

        //发货单
        DeliveryOrderEntity deliveryOrderEntity =
                deliveryOrderService.getOne(Wrappers.lambdaQuery(new DeliveryOrderEntity()).eq(DeliveryOrderEntity::getCode, deliveryOrderDetailEntity.getDoCode()));
        BeanUtils.build(reportDoInfo, deliveryOrderEntity);

        //库存分配
        StockAllocationEntity stockAllocationEntity =
                stockAllocationService.getOne(Wrappers.lambdaQuery(new StockAllocationEntity())
                        .eq(StockAllocationEntity::getDeliveryDetailId, deliveryOrderDetailEntity.getId())
                        .eq(StockAllocationEntity::getSkuCode, deliveryOrderDetailEntity.getSkuCode())
                        .eq(StockAllocationEntity::getDeliveryOrderId, deliveryOrderDetailEntity.getDeliveryOrderId())
                );
        BeanUtils.build(reportDoInfo, stockAllocationEntity);

        if (stockAllocationEntity != null) {
            //仓库
            WarehouseEntity warehouseEntity = warehouseService.getById(stockAllocationEntity.getWarehouseId());
            BeanUtils.build(reportDoInfo, warehouseEntity);

            //库位
            StockContainerEntity stockContainerEntity = stockContainerService.getById(stockAllocationEntity.getContainerId());
            BeanUtils.build(reportDoInfo, stockContainerEntity);
        }



        save(reportDoInfo);
        return reportDoInfo;
    }
}
